@@ -22,10 +22,6 @@ New in pygame 1.9.0.
 #        - that can automatically inject input into the event queue
 #          once the input object is running.  Like joysticks.
 
-
-# In Python 2.7 pygame.math is imported instead of the built-in math module.
-# This import from future allows the built-in math module to be imported.
-from __future__ import absolute_import
 import math
 import atexit
 
@@ -233,7 +229,7 @@ def get_default_output_id():
 
 
 def get_device_info(an_id):
-    """ returns information about a midi device
+    """returns information about a midi device
     pygame.midi.get_device_info(an_id): return (interf, name,
                                                 input, output,
                                                 opened)
@@ -250,7 +246,7 @@ def get_device_info(an_id):
     return _pypm.GetDeviceInfo(an_id)
 
 
-class Input(object):
+class Input:
     """Input is used to get midi input from midi devices.
     Input(device_id)
     Input(device_id, buffer_size)
@@ -267,9 +263,11 @@ class Input(object):
         _check_init()
 
         if device_id == -1:
-            raise MidiException("Device id is -1, not a valid output id.  "
-                                "-1 usually means there were no default "
-                                "Output devices.")
+            raise MidiException(
+                "Device id is -1, not a valid output id.  "
+                "-1 usually means there were no default "
+                "Output devices."
+            )
 
         try:
             result = get_device_info(device_id)
@@ -291,8 +289,8 @@ class Input(object):
 
             elif is_output:
                 raise MidiException(
-                    "Device id given is not a valid"
-                    " input id, it is an output id.")
+                    "Device id given is not a valid input id, it is an output id."
+                )
             else:
                 raise MidiException("Device id given is not a valid input id.")
         else:
@@ -303,7 +301,7 @@ class Input(object):
             raise MidiException("midi not open.")
 
     def close(self):
-        """ closes a midi stream, flushing any pending buffers.
+        """closes a midi stream, flushing any pending buffers.
         Input.close(): return None
 
         PortMidi attempts to close open streams when the application
@@ -346,7 +344,7 @@ class Input(object):
         raise MidiException((result, err_text))
 
 
-class Output(object):
+class Output:
     """Output is used to send midi to an output device
     Output(device_id)
     Output(device_id, latency = 0)
@@ -374,7 +372,7 @@ class Output(object):
 
     """
 
-    def __init__(self, device_id, latency=0, buffer_size=4096):
+    def __init__(self, device_id, latency=0, buffer_size=256):
         """Output(device_id)
         Output(device_id, latency = 0)
         Output(device_id, buffer_size = 4096)
@@ -404,9 +402,11 @@ class Output(object):
         self._aborted = 0
 
         if device_id == -1:
-            raise MidiException("Device id is -1, not a valid output id."
-                                "  -1 usually means there were no default "
-                                "Output devices.")
+            raise MidiException(
+                "Device id is -1, not a valid output id."
+                "  -1 usually means there were no default "
+                "Output devices."
+            )
 
         try:
             result = get_device_info(device_id)
@@ -421,17 +421,17 @@ class Output(object):
             _, _, is_input, is_output, _ = result
             if is_output:
                 try:
-                    self._output = _pypm.Output(device_id, latency)
+                    self._output = _pypm.Output(device_id, latency, buffer_size)
                 except TypeError:
                     raise TypeError("an integer is required")
                 self.device_id = device_id
 
             elif is_input:
-                raise MidiException("Device id given is not a valid output "
-                                    "id, it is an input id.")
+                raise MidiException(
+                    "Device id given is not a valid output id, it is an input id."
+                )
             else:
-                raise MidiException("Device id given is not a"
-                                    " valid output id.")
+                raise MidiException("Device id given is not a valid output id.")
         else:
             raise MidiException("Device id invalid, out of range.")
 
@@ -443,7 +443,7 @@ class Output(object):
             raise MidiException("midi aborted.")
 
     def close(self):
-        """ closes a midi stream, flushing any pending buffers.
+        """closes a midi stream, flushing any pending buffers.
         Output.close(): return None
 
         PortMidi attempts to close open streams when the application
@@ -521,7 +521,7 @@ class Output(object):
         Output.write_sys_ex(when, msg)
 
         msg - can be a *list* or a *string*
-        when - a timestamp in miliseconds
+        when - a timestamp in milliseconds
         example:
           (assuming o is an onput MIDI stream)
             o.write_sys_ex(0,'\\xF0\\x7D\\x10\\x11\\x12\\x13\\xF7')
@@ -572,12 +572,12 @@ class Output(object):
         Also called "patch change" or "program change".
         """
         if not 0 <= instrument_id <= 127:
-            raise ValueError("Undefined instrument id: %d" % instrument_id)
+            raise ValueError(f"Undefined instrument id: {instrument_id}")
 
         if not 0 <= channel <= 15:
             raise ValueError("Channel not between 0 and 15.")
 
-        self.write_short(0xc0 + channel, instrument_id)
+        self.write_short(0xC0 + channel, instrument_id)
 
     def pitch_bend(self, value=0, channel=0):
         """modify the pitch of a channel.
@@ -595,16 +595,17 @@ class Output(object):
             raise ValueError("Channel not between 0 and 15.")
 
         if not -8192 <= value <= 8191:
-            raise ValueError("Pitch bend value must be between "
-                             "-8192 and +8191, not %d." % value)
+            raise ValueError(
+                f"Pitch bend value must be between -8192 and +8191, not {value}."
+            )
 
         # "The 14 bit value of the pitch bend is defined so that a value of
         # 0x2000 is the center corresponding to the normal pitch of the note
         # (no pitch change)." so value=0 should send 0x2000
         value = value + 0x2000
-        lsb = value & 0x7f  # keep least 7 bits
+        lsb = value & 0x7F  # keep least 7 bits
         msb = value >> 7
-        self.write_short(0xe0 + channel, lsb, msb)
+        self.write_short(0xE0 + channel, lsb, msb)
 
 
 # MIDI commands
@@ -639,13 +640,15 @@ def midis2events(midis, device_id):
     for midi in midis:
         ((status, data1, data2, data3), timestamp) = midi
 
-        event = pygame.event.Event(MIDIIN,
-                                   status=status,
-                                   data1=data1,
-                                   data2=data2,
-                                   data3=data3,
-                                   timestamp=timestamp,
-                                   vice_id=device_id)
+        event = pygame.event.Event(
+            MIDIIN,
+            status=status,
+            data1=data1,
+            data2=data2,
+            data3=data3,
+            timestamp=timestamp,
+            vice_id=device_id,
+        )
         evs.append(event)
 
     return evs
@@ -657,7 +660,7 @@ class MidiException(Exception):
     """
 
     def __init__(self, value):
-        super(MidiException, self).__init__(value)
+        super().__init__(value)
         self.parameter = value
 
     def __str__(self):
@@ -665,7 +668,7 @@ class MidiException(Exception):
 
 
 def frequency_to_midi(frequency):
-    """ converts a frequency into a MIDI note.
+    """converts a frequency into a MIDI note.
 
     Rounds to the closest midi note.
 
@@ -678,17 +681,11 @@ def frequency_to_midi(frequency):
     >>> frequency_to_midi(4186.0)
     108
     """
-    return int(
-        round(
-            69 + (
-                12 * math.log(frequency / 440.0)
-            ) / math.log(2)
-        )
-    )
+    return int(round(69 + (12 * math.log(frequency / 440.0)) / math.log(2)))
 
 
 def midi_to_frequency(midi_note):
-    """ Converts a midi note to a frequency.
+    """Converts a midi note to a frequency.
 
     ::Examples::
 
@@ -699,11 +696,11 @@ def midi_to_frequency(midi_note):
     >>> midi_to_frequency(108)
     4186.0
     """
-    return round(440.0 * 2 ** ((midi_note - 69) * (1. / 12.)), 1)
+    return round(440.0 * 2 ** ((midi_note - 69) * (1.0 / 12.0)), 1)
 
 
 def midi_to_ansi_note(midi_note):
-    """ returns the Ansi Note name for a midi number.
+    """returns the Ansi Note name for a midi number.
 
     ::Examples::
 
@@ -714,8 +711,8 @@ def midi_to_ansi_note(midi_note):
     >>> midi_to_ansi_note(108)
     'C8'
     """
-    notes = ['A', 'A#', 'B', 'C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#']
+    notes = ["A", "A#", "B", "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#"]
     num_notes = 12
-    note_name = notes[int(((midi_note - 21) % num_notes))]
+    note_name = notes[int((midi_note - 21) % num_notes)]
     note_number = (midi_note - 12) // num_notes
-    return '%s%s' % (note_name, note_number)
+    return f"{note_name}{note_number}"
