@@ -1,5 +1,4 @@
 #################################### IMPORTS ###################################
-# -*- encoding: utf-8 -*-
 
 
 import unittest
@@ -1204,7 +1203,7 @@ class LayeredDirtyTypeTest__DirtySprite(LayeredGroupBase, unittest.TestCase):
 
                     color = surface.get_at((x, y))
 
-                    self.assertEqual(color, expected_color, "pos=({}, {})".format(x, y))
+                    self.assertEqual(color, expected_color, f"pos=({x}, {y})")
         finally:
             surface.unlock()
 
